// Pin definitions
int G1 = 13; // Green LED for light set 1
int Y1 = 12; // Yellow LED for light set 1
int R1 = 11; // Red LED for light set 1
int G2 = 10; // Green LED for light set 2
int Y2 = 9;  // Yellow LED for light set 2
int R2 = 8;  // Red LED for light set 2


int button1 = 7; // Button for light set 1
int button2 = 6; // Button for light set 2


// Timing intervals (in milliseconds)
unsigned long greenTime = 10000;  // Green light duration
unsigned long yellowTime = 1000; // Yellow light duration


unsigned long lastChange = 0;    // Tracks the last state change time
int state = 0;                   // Traffic light state (0: NS green, 1: NS yellow, 2: EW green, 3: EW yellow)


bool button1Pressed = false; // Flags for button press
bool button2Pressed = false;


void setup() {
  Serial.begin(9600);
  // Set LED pins as outputs
  pinMode(G1, OUTPUT);
  pinMode(Y1, OUTPUT);
  pinMode(R1, OUTPUT);
  pinMode(G2, OUTPUT);
  pinMode(Y2, OUTPUT);
  pinMode(R2, OUTPUT);


  // Set button pins as inputs with pull-up resistors
  pinMode(button1, INPUT_PULLUP);
  pinMode(button2, INPUT_PULLUP);


  // Initialize lights
  digitalWrite(G1, HIGH);
  digitalWrite(Y1, LOW);
  digitalWrite(R1, LOW);
  digitalWrite(G2, LOW);
  digitalWrite(Y2, LOW);
  digitalWrite(R2, HIGH);


  lastChange = millis();
}


void loop() {
  unsigned long currentTime = millis();


  // Read button states (active low)
  if (digitalRead(button1) == LOW) {
    button1Pressed = true;
    Serial.println("NS Button Press");
  }
  if (digitalRead(button2) == LOW) {
    button2Pressed = true;
     Serial.println("EW Button Press");
  }


  switch (state) {
    case 0: // NS Green, EW Red
        Serial.println("NS Green, EW Red");
      if (currentTime - lastChange >= greenTime || button2Pressed) {
        Serial.println("Transition to NS Yellow");
        digitalWrite(G1, LOW);
        digitalWrite(Y1, HIGH);
        state = 1;
        lastChange = currentTime;
        button2Pressed = false; // Reset the button flag
      }
      break;


    case 1: // NS Yellow, EW Red
          Serial.println("NS Yellow, EW Red");
      if (currentTime - lastChange >= yellowTime) {
        Serial.println("Transition to EW Green");
        digitalWrite(Y1, LOW);
        digitalWrite(R1, HIGH);
        digitalWrite(R2, LOW);
        digitalWrite(G2, HIGH);
        state = 2;
        lastChange = currentTime;
      }
      break;


    case 2: // NS Red, EW Green
      Serial.println("NS Red, EW Green");


      if (currentTime - lastChange >= greenTime || button1Pressed) {
        Serial.println("Transition to EW Yellow");
        digitalWrite(G2, LOW);
        digitalWrite(Y2, HIGH);
        state = 3;
        lastChange = currentTime;
        button1Pressed = false; // Reset the button flag
      }
      break;


    case 3: // NS Red, EW Yellow
            Serial.println("NS Red, EW Yellow");


      if (currentTime - lastChange >= yellowTime) {
        Serial.println("Transition to NS Green");
        digitalWrite(Y2, LOW);
        digitalWrite(R2, HIGH);
        digitalWrite(R1, LOW);
        digitalWrite(G1, HIGH);
        state = 0;
        lastChange = currentTime;
        
      }
      break;
  }
        button1Pressed = false; // Reset the button flag
        button2Pressed = false; // Reset the button flag


}